@@ -41,7 +41,6 @@ namespace GiNaC {
 GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(add, expairseq,
   print_func<print_context>(&add::do_print).
   print_func<print_latex>(&add::do_print_latex).
-  print_func<print_csrc>(&add::do_print_csrc).
   print_func<print_tree>(&add::do_print_tree).
   print_func<print_python_repr>(&add::do_print_python_repr))
 
@@ -189,48 +188,6 @@ void add::do_print_latex(const print_latex & c, unsigned level) const
 	print_add(c, level, true);
 }
 
-void add::do_print_csrc(const print_csrc & c, unsigned level) const
-{
-	if (precedence() <= level)
-		c.s << "(";
-	
-	// Print arguments, separated by "+" or "-"
-	char separator = ' ';
-        for (const auto & elem : seq) {
-		numeric co = ex_to<numeric>(elem.coeff);
-		// If the coefficient is negative, separator is "-"
-		if (co.is_minus_one()
-                    or co.numer().is_minus_one())
-			separator = '-';
-		c.s << separator;
-		if (co.is_one() or co.is_minus_one()) {
-			elem.rest.print(c, precedence());
-		} else if (co.numer().is_one()
-                        or co.numer().is_minus_one())
-		{
-			elem.rest.print(c, precedence());
-			c.s << '/';
-			ex_to<numeric>(elem.coeff).denom().print(c, precedence());
-		} else {
-			elem.coeff.print(c, precedence());
-			c.s << '*';
-			elem.rest.print(c, precedence());
-		}
-		
-		separator = '+';
-	}
-	
-	if (!overall_coeff.is_zero()) {
-		if (overall_coeff.info(info_flags::positive)
-		 || is_a<print_csrc_cl_N>(c) || !overall_coeff.info(info_flags::real))  // sign inside ctor argument
-			c.s << '+';
-		overall_coeff.print(c, precedence());
-	}
-		
-	if (precedence() <= level)
-		c.s << ")";
-}
-
 void add::do_print_python_repr(const print_python_repr & c, unsigned /*level*/) const
 {
 	c.s << class_name() << '(';
@@ -244,72 +201,77 @@ void add::do_print_python_repr(const print_python_repr & c, unsigned /*level*/)
 
 bool add::info(unsigned inf) const
 {
-	switch (inf) {
-		case info_flags::polynomial:
-		case info_flags::integer_polynomial:
-		case info_flags::cinteger_polynomial:
-		case info_flags::rational_polynomial:
-		case info_flags::real:
-		case info_flags::rational:
-		case info_flags::integer:
-		case info_flags::crational:
-		case info_flags::cinteger:
-		case info_flags::nonnegative:
-		case info_flags::posint:
-		case info_flags::nonnegint:
-		case info_flags::even:
-		case info_flags::crational_polynomial:
-		case info_flags::rational_function: {
-			for (const auto & elem : seq) {
-				if (!(recombine_pair_to_ex(elem).info(inf)))
-					return false;
-			}
-			if (overall_coeff.is_zero() && (inf == info_flags::positive || inf == info_flags::posint))
-                                return true;
-                        return overall_coeff.info(inf);
-		}
-		case info_flags::inexact:
-		case info_flags::algebraic: {
-                        if (overall_coeff.info(inf))
-                                return true;
-                        for (const auto & elem : seq) {
-				if ((recombine_pair_to_ex(elem).info(inf)))
-					return true;
-			}
-			return false;
-		}
-		case info_flags::positive: {
-                        bool positive_seen = overall_coeff.is_positive();
-                        if (not positive_seen and not overall_coeff.is_zero())
+        switch (inf) {
+        case info_flags::nonzero:
+                return is_positive()
+                or info(info_flags::negative);
+        case info_flags::polynomial:
+        case info_flags::integer_polynomial:
+        case info_flags::cinteger_polynomial:
+        case info_flags::rational_polynomial:
+        case info_flags::real:
+        case info_flags::rational:
+        case info_flags::integer:
+        case info_flags::crational:
+        case info_flags::cinteger:
+        case info_flags::nonnegative:
+        case info_flags::posint:
+        case info_flags::nonnegint:
+        case info_flags::even:
+        case info_flags::crational_polynomial:
+        case info_flags::rational_function: {
+                for (const auto &elem : seq) {
+                        if (!(recombine_pair_to_ex(elem).info(inf)))
                                 return false;
-			for (const auto & elem : seq) {
-				ex t = recombine_pair_to_ex(elem);
-                                bool is_pos = t.info(info_flags::positive);
-                                if (not is_pos
-                                    and not t.info(info_flags::nonnegative))
-					return false;
-                                else if (not positive_seen and is_pos)
-                                        positive_seen = true;
-			}
-                        return positive_seen;
                 }
-		case info_flags::negative: {
-                        bool negative_seen = overall_coeff.is_negative();
-                        if (not negative_seen and not overall_coeff.is_zero())
+                if (overall_coeff.is_zero()
+                    and (inf == info_flags::positive
+                         or inf == info_flags::posint))
+                        return true;
+                return overall_coeff.info(inf);
+        }
+        case info_flags::inexact:
+        case info_flags::algebraic: {
+                if (overall_coeff.info(inf))
+                        return true;
+                for (const auto &elem : seq) {
+                        if ((recombine_pair_to_ex(elem).info(inf)))
+                                return true;
+                }
+                return false;
+        }
+        case info_flags::positive: {
+                bool positive_seen = overall_coeff.is_positive();
+                if (not positive_seen and not overall_coeff.is_zero())
+                        return false;
+                for (const auto &elem : seq) {
+                        ex t = recombine_pair_to_ex(elem);
+                        bool is_pos = t.is_positive();
+                        if (not is_pos
+                        and not t.info(info_flags::nonnegative))
                                 return false;
-			for (const auto & elem : seq) {
-				ex t = recombine_pair_to_ex(elem);
-                                bool is_neg = t.info(info_flags::negative);
-                                if (not is_neg
-                                    and not t.is_zero())
-					return false;
-                                else if (not negative_seen and is_neg)
-                                        negative_seen = true;
-			}
-                        return negative_seen;
+                        if (not positive_seen and is_pos)
+                                positive_seen = true;
                 }
-	}
-	return inherited::info(inf);
+                return positive_seen;
+        }
+        case info_flags::negative: {
+                bool negative_seen = overall_coeff.is_negative();
+                if (not negative_seen and not overall_coeff.is_zero())
+                        return false;
+                for (const auto &elem : seq) {
+                        ex t = recombine_pair_to_ex(elem);
+                        bool is_neg = t.info(info_flags::negative);
+                        if (not is_neg
+                        and not t.is_zero())
+                                return false;
+                        if (not negative_seen and is_neg)
+                                negative_seen = true;
+                }
+                return negative_seen;
+        }
+        }
+        return inherited::info(inf);
 }
 
 bool add::is_polynomial(const ex & var) const
@@ -356,7 +318,7 @@ ex add::coeff(const ex & s, const ex & n) const
         for (const auto & elem : seq) {
 		ex restcoeff = elem.rest.coeff(s, n);
  		if (!restcoeff.is_zero()) {
-			coeffseq.push_back(expair(restcoeff, elem.coeff));
+			coeffseq.emplace_back(restcoeff, elem.coeff);
 		}
 	}
 
@@ -405,7 +367,8 @@ ex add::eval(int level) const
 	if (seq_size == 0) {
 		// +(;c) -> c
 		return overall_coeff;
-	} else if (seq_size == 1 && overall_coeff.is_zero()) {
+	}
+        if (seq_size == 1 && overall_coeff.is_zero()) {
 		// +(x;0) -> x
 		return recombine_pair_to_ex(*(seq.begin()));
 	} else if (!overall_coeff.is_zero() && seq[0].rest.return_type() != return_types::commutative) {
@@ -468,8 +431,8 @@ ex add::conjugate() const
 	epvector v;
 	v.reserve(seq.size());
 	for (const auto & elem : seq)
-		if ((elem.coeff).info(info_flags::real)
-		    and (elem.rest).info(info_flags::real)) {
+		if ((elem.coeff).is_real()
+		    and (elem.rest).is_real()) {
 			v.push_back(elem);
 		} else {
 			ex cj=recombine_pair_to_ex(elem).conjugate();
@@ -485,10 +448,10 @@ ex add::real_part() const
 	epvector v;
 	v.reserve(seq.size());
 	for (const auto & elem : seq)
-		if ((elem.coeff).info(info_flags::real)) {
+		if ((elem.coeff).is_real()) {
 			ex rp = (elem.rest).real_part();
 			if (!rp.is_zero())
-				v.push_back(expair(rp, elem.coeff));
+				v.emplace_back(rp, elem.coeff);
 		} else {
 			ex rp=recombine_pair_to_ex(elem).real_part();
 			if (!rp.is_zero())
@@ -503,10 +466,10 @@ ex add::imag_part() const
 	epvector v;
 	v.reserve(seq.size());
 	for (const auto & elem : seq)
-		if ((elem.coeff).info(info_flags::real)) {
+		if ((elem.coeff).is_real()) {
 			ex ip = (elem.rest).imag_part();
 			if (!ip.is_zero())
-				v.push_back(expair(ip, elem.coeff));
+				v.emplace_back(ip, elem.coeff);
 		} else {
 			ex ip=recombine_pair_to_ex(elem).imag_part();
 			if (!ip.is_zero())
@@ -529,7 +492,7 @@ ex add::derivative(const symbol & y) const
 	// than the default implementation in basic::derivative() although
 	// if performs the same function (differentiate each term).
 	for (const auto & elem : seq)
-		s.push_back(expair(elem.rest.diff(y), elem.coeff));
+		s.emplace_back(elem.rest.diff(y), elem.coeff);
 	return (new add(s, *_num0_p))->setflag(status_flags::dynallocated);
 }
 
@@ -542,16 +505,16 @@ unsigned add::return_type() const
 {
 	if (seq.empty())
 		return return_types::commutative;
-	else
-		return seq.begin()->rest.return_type();
+	
+        return seq.begin()->rest.return_type();
 }
 
 tinfo_t add::return_type_tinfo() const
 {
 	if (seq.empty())
 		return this;
-	else
-		return seq.begin()->rest.return_type_tinfo();
+	
+        return seq.begin()->rest.return_type_tinfo();
 }
 
 // Note: do_index_renaming is ignored because it makes no sense for an add.
@@ -598,9 +561,7 @@ expair add::combine_ex_with_coeff_to_pair(const ex & e,
 		mulcopyp->setflag(status_flags::dynallocated);
 		if (c.is_one())
 			return expair(*mulcopyp, numfactor);
-		else
-			return expair(*mulcopyp,
-                                        numfactor*c);
+                return expair(*mulcopyp, numfactor*c);
 	} else if (is_exactly_a<numeric>(e)) {
 		if (c.is_one())
 			return expair(e, _ex1);
@@ -618,7 +579,7 @@ expair add::combine_pair_with_coeff_to_pair(const expair & p,
 
 	if (is_exactly_a<numeric>(p.rest)) {
 		GINAC_ASSERT(ex_to<numeric>(p.coeff).is_one()); // should be normalized
-		return expair(ex_to<numeric>(p.rest).mul_dyn(c),_ex1);
+                return expair(ex_to<numeric>(p.rest).mul_dyn(c),_ex1);
 	}
 
 	return expair(p.rest,ex_to<numeric>(p.coeff).mul_dyn(c));
@@ -628,8 +589,118 @@ ex add::recombine_pair_to_ex(const expair & p) const
 {
 	if (ex_to<numeric>(p.coeff).is_one())
 		return p.rest;
-	else
-		return (new mul(p.rest,p.coeff))->setflag(status_flags::dynallocated);
+	
+        return (new mul(p.rest,p.coeff))->setflag(status_flags::dynallocated);
+}
+
+ex add::power(const numeric& expo) const
+{
+        using POW = class power;
+        numeric icont = integer_content();
+        const numeric lcoeff = ex_to<numeric>(lead_coeff()).div(icont);
+        if (not lcoeff.is_positive())
+                icont = icont.negative();
+        if (icont.is_one()
+            or not lcoeff.is_integer())
+                return (new POW(*this, expo))->
+                        setflag(status_flags::dynallocated
+                                | status_flags::evaluated);
+
+        ex c;
+        if (expo.is_integer()) {
+                c = icont.pow_intexp(expo.to_long());
+        }
+        else {
+                bool ppower_equals_one;
+                numeric newbasis, ppower;
+                rational_power_parts(icont,
+                                     expo,
+                                     ppower,
+                                     newbasis,
+                                     ppower_equals_one);
+                if (not newbasis.is_one()
+                    and (ppower_equals_one or icont.is_negative()))
+                        return (new POW(*this, expo))->
+                                setflag(status_flags::dynallocated
+                                      | status_flags::evaluated);
+                if (ppower_equals_one)
+                        c = *_num1_p;
+                else
+                        c = ppower * POW(newbasis, expo).hold();
+        }
+
+        auto addp = new add(*this);
+        addp->setflag(status_flags::dynallocated);
+        addp->clearflag(status_flags::hash_calculated);
+        addp->overall_coeff /= icont;
+        addp->seq_sorted.resize(0);
+        for (auto &elem : addp->seq)
+                elem.coeff = ex_to<numeric>(elem.coeff).div_dyn(icont);
+        if (likely(not c.is_one()))
+                return (new mul(POW(*addp, expo), c))->
+                        setflag(status_flags::dynallocated
+                                | status_flags::evaluated);
+        return (new POW(*addp, expo))->
+                setflag(status_flags::dynallocated
+                        | status_flags::evaluated);
+}
+
+// This is an alternative version of add::power that is necessary to
+// canonicalize powers of sums (always draws out the positive content).
+// It would be difficult to introduce this as general behaviour in Sage.
+ex Power(const ex& base_, const ex& expo_)
+{
+        if (not is_exactly_a<add>(base_)
+            or not is_exactly_a<numeric>(expo_))
+                return power(base_, expo_);
+        using POW = class power;
+        const add& base = ex_to<add>(base_);
+        const numeric& expo = ex_to<numeric>(expo_);
+        numeric icont = base.integer_content();
+        const numeric lcoeff = ex_to<numeric>(base.lead_coeff()).div(icont);
+        if (icont.is_one()
+            or icont.is_minus_one()
+            or not lcoeff.is_integer())
+                return (new POW(base, expo))->
+                        setflag(status_flags::dynallocated
+                                | status_flags::evaluated);
+
+        ex c;
+        if (expo.is_integer()) {
+                c = icont.pow_intexp(expo.to_long());
+        }
+        else {
+                bool ppower_equals_one;
+                numeric newbasis, ppower;
+                rational_power_parts(icont,
+                                     expo,
+                                     ppower,
+                                     newbasis,
+                                     ppower_equals_one);
+                if (ppower_equals_one) {
+                        c = POW(icont.abs(), expo).hold();
+                }
+                else
+                        if (icont.is_negative())
+                                c = ppower * POW(-newbasis, expo).hold();
+                        else
+                                c = ppower * POW(newbasis, expo).hold();
+        }
+
+        auto addp = new add(base);
+        addp->setflag(status_flags::dynallocated);
+        addp->clearflag(status_flags::hash_calculated);
+        addp->overall_coeff /= icont.abs();
+        addp->seq_sorted.resize(0);
+        for (auto &elem : addp->seq)
+                elem.coeff = ex_to<numeric>(elem.coeff).div_dyn(icont.abs());
+        if (likely(not c.is_one()))
+                return (new mul(POW(*addp, expo), c))->
+                        setflag(status_flags::dynallocated
+                                | status_flags::evaluated);
+        return (new POW(*addp, expo))->
+                setflag(status_flags::dynallocated
+                        | status_flags::evaluated);
 }
 
 ex add::expand(unsigned options) const
@@ -659,9 +730,9 @@ ex add::lead_coeff() const {
 	if (seq_sorted.empty() and not seq.empty()) {
 		return min_element(seq.begin(), seq.end(),
 				print_order_pair())->coeff;
-	} else {
+	} 
 		return seq_sorted.begin()->coeff;
-	}
+	
 }
 
 ex add::combine_fractions() const
@@ -670,9 +741,10 @@ ex add::combine_fractions() const
         exmap fmap;
         ex oc = overall_coeff;
         for (const auto& pair : seq) {
+                using POW = class power;
                 const ex &e = recombine_pair_to_ex(pair);
-                if (is_exactly_a<power>(e)) {
-                        const power& p = ex_to<power>(e);
+                if (is_exactly_a<POW>(e)) {
+                        const POW& p = ex_to<POW>(e);
                         const ex& eexp = p.op(1);
                         if (eexp.info(info_flags::negative)) {
                                 auto it = fmap.find(p);
