@@ -20,6 +20,7 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#define register
 #include <Python.h>
 #include "ex.h"
 #include "ex_utils.h"
@@ -35,6 +36,7 @@
 #include "relational.h"
 #include "utils.h"
 #include "operators.h"
+#include "wildcard.h"
 
 #include <iostream>
 #include <stdexcept>
@@ -80,8 +82,8 @@ ex ex::expand(unsigned options) const
 {
 	if (options == 0 && ((bp->flags & status_flags::expanded) != 0u)) // The "expanded" flag only covers the standard options; someone might want to re-expand with different options
 		return *this;
-	else
-		return bp->expand(options);
+
+        return bp->expand(options);
 }
 
 /** Compute partial derivative of an expression.
@@ -93,15 +95,48 @@ ex ex::diff(const symbol & s, unsigned nth) const
 {
 	if (nth == 0u)
 		return *this;
-	else
-		return bp->diff(s, nth);
+
+	return bp->diff(s, nth);
 }
 
 /** Check whether expression matches a specified pattern. */
 bool ex::match(const ex & pattern) const
 {
-	lst repl_lst;
-	return bp->match(pattern, repl_lst);
+	exmap map;
+	return bp->match(pattern, map);
+}
+
+bool ex::match(const ex & pattern, lst & repl_lst) const
+{
+        exmap map;
+        bool ret = bp->match(pattern, map);
+        for (const auto& pair : map)
+                repl_lst.append(pair.first == pair.second);
+        return ret;
+}
+
+bool ex::match(const ex & pattern, exvector& vec) const
+{
+        exmap map;
+        bool ret = this->match(pattern, map);
+        if (not ret)
+                return ret;
+        unsigned maxl = 0;
+        for (const auto& pair : map) {
+                if (not is_exactly_a<wildcard>(pair.first))
+                        throw std::runtime_error("no wildcard");
+                unsigned l = ex_to<wildcard>(pair.first).get_label();
+                if (maxl < l)
+                        maxl = l;
+        }
+        ex nan = NaN;
+        exvector tvec(maxl+1);
+        tvec.assign(maxl+1, nan);
+        for (const auto& pair : map) {
+                tvec[ex_to<wildcard>(pair.first).get_label()] = pair.second;
+        }
+        vec = tvec;
+        return ret;
 }
 
 /** Find all occurrences of a pattern. The found matches are appended to
@@ -165,12 +200,14 @@ ex ex::subs(const ex & e, unsigned options) const
 
 		return bp->subs(m, options);
 
-	} else if (e.info(info_flags::list)) {
+	}
+        
+        if (e.info(info_flags::list)) {
 
 		// Argument is a list: convert it to a map
 		exmap m;
 		GINAC_ASSERT(is_a<lst>(e));
-		for (auto r : ex_to<lst>(e)) {
+		for (const auto& r : ex_to<lst>(e)) {
 			
 			if (!r.info(info_flags::relation_equal))
 				throw(std::invalid_argument("basic::subs(ex): argument must be a list of equations"));
@@ -191,6 +228,17 @@ ex ex::subs(const ex & e, unsigned options) const
 		throw(std::invalid_argument("ex::subs(ex): argument must be a relation_equal or a list"));
 }
 
+ex ex::subs(const exmap& m, unsigned options) const
+{
+        if ((options & subs_options::no_pattern) != 0)
+                return bp->subs(m, options);
+        for (const auto& p : m)
+                if (haswild(p.first))
+                        return bp->subs(m, options);
+
+        return bp->subs(m, options | subs_options::no_pattern);
+}
+
 /** Traverse expression tree with given visitor, preorder traversal. */
 void ex::traverse_preorder(visitor & v) const
 {
@@ -304,8 +352,8 @@ bool ex::is_polynomial(const ex & vars) const
 				return false;
 		return true;
 	}
-	else
-		return bp->is_polynomial(vars);
+
+	return bp->is_polynomial(vars);
 }
 
 bool ex::is_zero() const {
@@ -317,23 +365,47 @@ bool ex::is_zero() const {
         return ex_to<numeric>(*this).is_zero();
 }
 
-bool ex::is_integer_one() const
+bool ex::is_one() const
 {
-    if (!is_exactly_a<numeric>(*this))
-        return false;
-    const numeric& num = ex_to<numeric>(*this);
-    return num.is_integer() and num.is_one();
+        if (!is_exactly_a<numeric>(*this))
+                return false;
+        const numeric& num = ex_to<numeric>(*this);
+        return num.is_one();
 }
   
-bool ex::is_integer_pmone() const
+bool ex::is_minus_one() const
 {
-    if (!is_exactly_a<numeric>(*this))
-        return false;
-    const numeric& num = ex_to<numeric>(*this);
-    return num.is_integer()
-           and (num.is_one() or num.is_minus_one());
+        if (!is_exactly_a<numeric>(*this))
+                return false;
+        const numeric& num = ex_to<numeric>(*this);
+        return num.is_minus_one();
 }
 
+bool ex::is_negative_or_minus() const
+{
+        if (is_exactly_a<mul>(*this)
+           and ex_to<mul>(*this).get_overall_coeff().is_negative())
+                return true;
+        return is_negative();
+}
+
+bool ex::is_num_integer() const
+{
+        if (!is_exactly_a<numeric>(*this))
+                return false;
+        const numeric& num = ex_to<numeric>(*this);
+        return num.is_integer();
+}
+
+bool ex::is_num_fraction() const
+{
+        if (!is_exactly_a<numeric>(*this))
+                return false;
+        const numeric& num = ex_to<numeric>(*this);
+        return num.is_mpq();
+}
+
+
 void ex::set_domain(unsigned d)
 {
         if (is_exactly_a<symbol>(*this)) {
@@ -346,6 +418,20 @@ void ex::set_domain(unsigned d)
         }
 }
 
+static void _treesize(const ex& e, size_t& n)
+{
+        ++n;
+        for (size_t i=0; i < e.nops(); i++)
+                _treesize(e.op(i), n);
+}
+
+size_t ex::treesize() const
+{
+        size_t n = 0;
+        _treesize(*this, n);
+        return n;
+}
+
 size_t ex::nsymbols() const
 // DON'T USE if you want the number of symbols, instead use symbols().size()
 {
@@ -371,7 +457,8 @@ bool ex::get_first_symbol(ex &x) const
 	if (is_exactly_a<symbol>(*this)) {
 		x = *this;
 		return true;
-	} else if (is_exactly_a<add>(*this) || is_exactly_a<mul>(*this)) {
+	} 
+        if (is_exactly_a<add>(*this) || is_exactly_a<mul>(*this)) {
 		for (size_t i=0; i<nops(); i++)
 			if (sorted_op(i).get_first_symbol(x))
 				return true;
@@ -446,12 +533,45 @@ symbolset ex::free_symbols() const
 	return the_set;
 }
 
+static void collect_functions(const ex& e, std::unordered_set<unsigned>& funs)
+{
+	if (is_exactly_a<function>(e)) {
+                const function& f = ex_to<function>(e);
+                funs.insert(f.get_serial());
+        }
+        for (size_t i=0; i < e.nops(); i++)
+                collect_functions(e.op(i), funs);
+}
+
+std::unordered_set<unsigned> ex::functions() const
+{
+        std::unordered_set<unsigned> the_set;
+        collect_functions(*this, the_set);
+        return the_set;
+}
+
+static void collect_wilds(const ex& e, wildset& wilds)
+{
+	if (is_exactly_a<wildcard>(e))
+		wilds.insert(ex_to<wildcard>(e));
+	else
+		for (size_t i=0; i < e.nops(); i++)
+                        collect_wilds(e.op(i), wilds);
+}
+
+wildset ex::wilds() const
+{
+        wildset the_set;
+        collect_wilds(*this, the_set);
+	return the_set;
+}
+
 ex ex::sorted_op(size_t i) const
 {
 	if (is_a<expairseq>(*this))
 		return dynamic_cast<const expairseq&>(*bp).stable_op(i);
-	else
-		return bp->op(i);
+
+        return bp->op(i);
 }
 
 static bool has_nonposint_power(const ex& x, const symbol& symb)
@@ -490,20 +610,19 @@ static bool match_monom(const ex& term, const symbol& symb,
                         vec.push_back(std::make_pair(_ex1, expo.subs(map)));
                         return true;
                 }
-                else {
-                        // of form (...+...)^expo
-                        // we expand only those with integer exponent
-                        if (is_exactly_a<numeric>(expo)
-                                        and has_free_symbol(p.op(0), symb)) {
-                                numeric ee = ex_to<numeric>(expo);
-                                if (ee.is_integer() and ee.to_int() > 1) {
-                                        expairvec tmpvec;
-                                        expand(term).coefficients(symb, tmpvec);
-                                        for (const auto& pair : tmpvec)
-                                                vec.push_back(std::make_pair(pair.first.subs(map),
-                                                                        pair.second.subs(map)));
-                                        return true;
-                                }
+
+                // of form (...+...)^expo
+                // we expand only those with integer exponent
+                if (is_exactly_a<numeric>(expo)
+                                and has_free_symbol(p.op(0), symb)) {
+                        const numeric& ee = ex_to<numeric>(expo);
+                        if (ee.is_integer() and ee.to_int() > 1) {
+                                expairvec tmpvec;
+                                expand(term).coefficients(symb, tmpvec);
+                                for (const auto& pair : tmpvec)
+                                        vec.push_back(std::make_pair(pair.first.subs(map),
+                                                                pair.second.subs(map)));
+                                return true;
                         }
                 }
         }
@@ -536,7 +655,10 @@ static bool match_monom(const ex& term, const symbol& symb,
                                 numeric ee = ex_to<numeric>(mterm.op(1));
                                 if (ee.is_integer() and ee.to_int() > 1) {
                                         expairvec tmpvec;
-                                        expand(term).coefficients(symb, tmpvec);
+                                        ex e = expand(term);
+                                        if (not is_exactly_a<add>(e))
+                                                return false;
+                                        e.coefficients(symb, tmpvec);
                                         for (const auto& pair : tmpvec)
                                                 vec.push_back(std::make_pair(pair.first.subs(map),
                                                         pair.second.subs(map)));
@@ -567,6 +689,80 @@ numeric ex::ldegree(const ex & s) const
         return bp->ldegree(s);
 }
 
+static bool _collect_powers(ex& e, ex& repl, bool& collected)
+{
+        bool is_changed = false;
+        if (is_exactly_a<symbol>(e)
+            or is_exactly_a<numeric>(e)
+            or is_exactly_a<constant>(e))
+                return is_changed;
+        if (is_a<expairseq>(e)) {
+                for (size_t i=0; i<e.nops(); ++i) {
+                        ex ee, et = e.op(i);
+                        bool c = false;
+                        bool changed = _collect_powers(et, ee, c);
+                        if (changed or c)
+                                is_changed = true;
+                        if (c)
+                                e.set_epseq_from(i, ee);
+                }
+        }
+        else {
+                for (size_t i=0; i<e.nops(); ++i) {
+                        ex ee;
+                        bool c = false;
+                        bool changed = _collect_powers(e.let_op(i), ee, c);
+                        if (changed or c)
+                                is_changed = true;
+                        if (c)
+                                e.let_op(i) = ee;
+                }
+        }
+
+        if (not is_exactly_a<mul>(e))
+                return is_changed;
+        exmap map;
+        for (auto term : e) {
+                if (is_exactly_a<power>(term)) {
+                        auto it = map.find(term.op(0));
+                        if (it == map.end())
+                                map.insert(std::make_pair(term.op(0),
+                                                          term.op(1)));
+                        else {
+                                it->second += term.op(1);
+                                collected = true;
+                        }
+                }
+                else {
+                        auto it = map.find(term);
+                        if (it == map.end())
+                                map.insert(std::make_pair(term, _ex1));
+                        else {
+                                it->second += _ex1;
+                                collected = true;
+                        }
+                }
+        }
+        if (not collected)
+                return is_changed;
+        exvector vec;
+        for (auto p : map)
+                vec.push_back(power(p.first, p.second));
+        repl = mul(vec);
+        return is_changed;
+}
+
+// Repeatedly: for any mul in the ex, add exponents of powers
+// with the same base
+ex ex::collect_powers() const
+{
+        ex the_ex = *this;
+        bool b;
+        ex r;
+        (void)_collect_powers(the_ex, r, b);
+        return b? r:the_ex;
+}
+
 /**
  * Return in vec a list of pairs with (coefficient, exponent) of ex
  * when interpreted as polynomial in s (with s any expression).
@@ -587,24 +783,21 @@ void ex::coefficients(const ex & s, expairvec & vec) const
         exmap submap {{s, symb}}, revmap {{symb, s}};
         ex sub = subs(submap);
 
-        if (has_nonposint_power(sub, symb)) {
-                vec.push_back(std::make_pair(*this, _ex0));
-                return;
-        }
-
         if (is_exactly_a<add>(sub)) {
                 const add& addref = ex_to<add>(sub);
                 const numeric& oc = addref.get_overall_coeff();
                 if (not oc.is_zero())
-                        vec.push_back(std::make_pair(oc, _ex0));
+                        vec.emplace_back(std::make_pair(oc, _ex0));
                 for (const auto& term : addref.seq) {
                         ex tmp = addref.recombine_pair_to_ex(term);
-                        if (not match_monom(tmp, symb, vec, revmap))
+                        if (has_nonposint_power(tmp, symb)
+                            or not match_monom(tmp, symb, vec, revmap))
                                 vec.push_back(std::make_pair(tmp.subs(revmap), _ex0));
                 }
         }
         else {
-                if (not match_monom(sub, symb, vec, revmap)) {
+                if (has_nonposint_power(sub, symb)
+                    or not match_monom(sub, symb, vec, revmap)) {
                         vec.clear();
                         vec.push_back(std::make_pair(*this, _ex0));
                 }
@@ -660,29 +853,27 @@ ex ex::deep_combine_fractions(ex e)
                                 is_exactly_a<constant>(e) or
                                 is_exactly_a<numeric>(e))
                         return e;
-                else
-                        for (unsigned int i=0; i<e.nops(); ++i) {
-                                e.let_op(i) = deep_combine_fractions(e.op(i));
-                        }
+                
+                for (unsigned int i=0; i<e.nops(); ++i) {
+                        e.let_op(i) = deep_combine_fractions(e.op(i));
+                }
         }
 
         if (is_exactly_a<add>(e)) {
                 ex t = ex_to<add>(e).combine_fractions();
                 return t;
         }
-        else
-                return e;
+
+        return e;
 }
 
 ex ex::combine_fractions(bool deep) const
 {
-        if (not deep) {
-                if (is_exactly_a<add>(*this))
-                        return ex_to<add>(*this).combine_fractions();
-                else
-                        return *this;
-        }
-        return deep_combine_fractions(*this);
+        if (deep)
+                return deep_combine_fractions(*this);
+        if (is_exactly_a<add>(*this))
+                return ex_to<add>(*this).combine_fractions();
+        return *this;
 }
 
 // private
@@ -748,26 +939,22 @@ ptr<basic> ex::construct_from_basic(const basic & other)
 		// soon as we leave the function, which would deallocate the
 		// evaluated object.
 		return tmpex.bp;
+	} 
 
-	} else {
+        // The easy case: making an "ex" out of an evaluated object.
+        if ((other.flags & status_flags::dynallocated) != 0u) {
 
-		// The easy case: making an "ex" out of an evaluated object.
-		if ((other.flags & status_flags::dynallocated) != 0u) {
+                // The object is already heap-allocated, so we can just make
+                // another reference to it.
+                return ptr<basic>(const_cast<basic &>(other));
+        } 
 
-			// The object is already heap-allocated, so we can just make
-			// another reference to it.
-			return ptr<basic>(const_cast<basic &>(other));
-
-		} else {
-
-			// The object is not heap-allocated, so we create a duplicate
-			// on the heap.
-			basic *bp = other.duplicate();
-			bp->setflag(status_flags::dynallocated);
-			GINAC_ASSERT(bp->get_refcount() == 0);
-			return bp;
-		}
-	}
+        // The object is not heap-allocated, so we create a duplicate
+        // on the heap.
+        basic *bp = other.duplicate();
+        bp->setflag(status_flags::dynallocated);
+        GINAC_ASSERT(bp->get_refcount() == 0);
+        return bp;
 }
 
 basic & ex::construct_from_int(int i)
