@@ -24,7 +24,7 @@
 
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wdeprecated-register"
-#include <factory/factory.h>
+#include "factory/factory.h"
 #pragma clang diagnostic pop
 
 #include "upoly.h"
@@ -48,7 +48,7 @@
 
 namespace GiNaC {
 
-void Log(const power_ocvector_map& m, std::string str)
+void Log(const power_ocvector_map& m, const std::string& str)
 {
         if (not str.empty())
                 std::cerr << str << ":\n";
@@ -107,8 +107,7 @@ static CanonicalForm num2canonical(const numeric& n, ex_int_map& map, exvector&
                 }
                 if (n.is_positive())
                         return replace_with_symbol(n, map, revmap);
-                else
-                        return -replace_with_symbol(n.negative(), map, revmap);
+                return -replace_with_symbol(n.negative(), map, revmap);
         }
 }
 
@@ -187,7 +186,7 @@ static void transform_powers(power_ocvector_map& pomap)
 }
 
 // Convert to Singular polynomial over QQ, filling replacement dicts
-const CanonicalForm ex::to_canonical(ex_int_map& map,
+const CanonicalForm ex::to_canonical(ex_int_map& amap,
                 power_ocvector_map& pomap,
                 exvector& revmap) const
 {
@@ -196,36 +195,36 @@ const CanonicalForm ex::to_canonical(ex_int_map& map,
                 const add& a = ex_to<add>(*this);
                 CanonicalForm p(0);
                 for (const auto& termex : a.seq) {
-                        p = p + a.recombine_pair_to_ex(termex).to_canonical(map, pomap, revmap);
+                        p = p + a.recombine_pair_to_ex(termex).to_canonical(amap, pomap, revmap);
                 }
-                p = p + num2canonical(a.overall_coeff, map, revmap);
+                p = p + num2canonical(a.overall_coeff, amap, revmap);
                 return p;
         }
-        else if (is_exactly_a<numeric>(*this))
+        if (is_exactly_a<numeric>(*this))
         {
-                return num2canonical(ex_to<numeric>(*this), map, revmap);
+                return num2canonical(ex_to<numeric>(*this), amap, revmap);
         }
-        else if (is_exactly_a<mul>(*this))
+        if (is_exactly_a<mul>(*this))
         {
                 const mul& m = ex_to<mul>(*this);
-                CanonicalForm p = num2canonical(*_num1_p, map, revmap);
+                CanonicalForm p = num2canonical(*_num1_p, amap, revmap);
                 for (const auto& termex : m.seq) {
-                        p = p * m.recombine_pair_to_ex(termex).to_canonical(map, pomap, revmap);
+                        p = p * m.recombine_pair_to_ex(termex).to_canonical(amap, pomap, revmap);
                 }
-                CanonicalForm oc = num2canonical(m.overall_coeff, map, revmap);
+                CanonicalForm oc = num2canonical(m.overall_coeff, amap, revmap);
                 p = p * oc;
                 return p;
         }
-        else if (is_exactly_a<power>(*this))
+        if (is_exactly_a<power>(*this))
         {
                 const power& pow = ex_to<power>(*this);
                 if (is_exactly_a<numeric>(pow.exponent)) {
-                        numeric expo = ex_to<numeric>(pow.exponent);
+                        const numeric& expo = ex_to<numeric>(pow.exponent);
                         if (expo.is_rational()) {
                                 CanonicalForm var;
                                 power_ocvector_map::iterator it;
                                 numeric n;
-                                var = replace_with_symbol(pow.basis, map, revmap);
+                                var = replace_with_symbol(pow.basis, amap, revmap);
                                 it = pomap.find(pow.basis);
                                 if (it == pomap.end())
                                         throw std::runtime_error("can't happen in ex::to_canonical");
@@ -257,7 +256,7 @@ const CanonicalForm ex::to_canonical(ex_int_map& map,
                         if (it == pomap.end())
                                 throw std::runtime_error("can't happen in ex::to_canonical");
                         CanonicalForm var = replace_with_symbol(it->first,
-                                                            map, revmap);
+                                                            amap, revmap);
                         numeric n = oc.div(it->second[0]);
                         ex b = it->first.subs(symbol_E == exp(1));
                         revmap[var.level()-1] = GiNaC::power(b, it->second[0]);
@@ -268,10 +267,10 @@ const CanonicalForm ex::to_canonical(ex_int_map& map,
                                 throw std::runtime_error("exponent too big");
                         }
                 }
-                return replace_with_symbol(*this, map, revmap);
+                return replace_with_symbol(*this, amap, revmap);
         }
 
-        return replace_with_symbol(*this, map, revmap);
+        return replace_with_symbol(*this, amap, revmap);
 }
 
 static numeric can2num(const CanonicalForm& f)
@@ -335,9 +334,7 @@ static ex canonical_to_ex(const CanonicalForm& f, const exvector& revmap)
                         numeric d(bigintden);
                         return n/d;
                 }
-                else {
-                        throw std::runtime_error("can't happen in canonical_to_ex #2");
-                }
+                throw std::runtime_error("can't happen in canonical_to_ex #2");
         }
 
         ex e = _ex0;
@@ -397,7 +394,8 @@ factored_a:
 		if (cb != nullptr)
 			*cb = part_b;
 		return (new mul(g))->setflag(status_flags::dynallocated);
-	} else if (is_exactly_a<mul>(b)) {
+	}
+        if (is_exactly_a<mul>(b)) {
 		if (is_exactly_a<mul>(a) && a.nops() > b.nops())
 			goto factored_a;
 factored_b:
@@ -433,39 +431,39 @@ factored_b:
 					if (cb != nullptr)
 						*cb = power(p, exp_b - exp_a);
 					return power(p, exp_a);
-				} else {
-					if (ca != nullptr)
-						*ca = power(p, exp_a - exp_b);
-					if (cb != nullptr)
-						*cb = _ex1;
-					return power(p, exp_b);
-				}
-			} else {
-				ex p_co, pb_co;
-				ex p_gcd = gcdpoly(p, pb, &p_co, &pb_co, check_args);
-				if (p_gcd.is_equal(_ex1)) {
-					// a(x) = p(x)^n, b(x) = p_b(x)^m, gcd (p, p_b) = 1 ==>
-					// gcd(a,b) = 1
-					if (ca != nullptr)
-						*ca = a;
-					if (cb != nullptr)
-						*cb = b;
-					return _ex1;
-					// XXX: do I need to check for p_gcd = -1?
-				} else {
-					// there are common factors:
-					// a(x) = g(x)^n A(x)^n, b(x) = g(x)^m B(x)^m ==>
-					// gcd(a, b) = g(x)^n gcd(A(x)^n, g(x)^(n-m) B(x)^m
-					if (exp_a < exp_b) {
-						return power(p_gcd, exp_a)*
-							gcdpoly(power(p_co, exp_a), power(p_gcd, exp_b-exp_a)*power(pb_co, exp_b), ca, cb, false);
-					} else {
-						return power(p_gcd, exp_b)*
-							gcdpoly(power(p_gcd, exp_a - exp_b)*power(p_co, exp_a), power(pb_co, exp_b), ca, cb, false);
-					}
-				} // p_gcd.is_equal(_ex1)
-			} // p.is_equal(pb)
+				} 
+                                if (ca != nullptr)
+                                        *ca = power(p, exp_a - exp_b);
+                                if (cb != nullptr)
+                                        *cb = _ex1;
+                                return power(p, exp_b);
+			}
 
+                        ex p_co, pb_co;
+                        ex p_gcd = gcdpoly(p, pb, &p_co, &pb_co, check_args);
+                        if (p_gcd.is_one()) {
+                                // a(x) = p(x)^n, b(x) = p_b(x)^m, gcd (p, p_b) = 1 ==>
+                                // gcd(a,b) = 1
+                                if (ca != nullptr)
+                                        *ca = a;
+                                if (cb != nullptr)
+                                        *cb = b;
+                                return _ex1;
+                                // XXX: do I need to check for p_gcd = -1?
+                        } 
+                        // there are common factors:
+                        // a(x) = g(x)^n A(x)^n, b(x) = g(x)^m B(x)^m ==>
+                        // gcd(a, b) = g(x)^n gcd(A(x)^n, g(x)^(n-m) B(x)^m
+                        if (exp_a < exp_b) {
+                                return power(p_gcd, exp_a)*
+                                        gcdpoly(power(p_co, exp_a),
+                                                power(p_gcd, exp_b-exp_a)*power(pb_co, exp_b),
+                                                ca, cb, false);
+                        } 
+                        return power(p_gcd, exp_b)*
+                                gcdpoly(power(p_gcd, exp_a - exp_b)*power(p_co, exp_a),
+                                        power(pb_co, exp_b),
+                                        ca, cb, false);
 		} else {
 			if (p.is_equal(b)) {
 				// a = p^n, b = p, gcd = p
@@ -479,17 +477,18 @@ factored_b:
 			ex p_co, bpart_co;
 			ex p_gcd = gcdpoly(p, b, &p_co, &bpart_co, false);
 
-			if (p_gcd.is_equal(_ex1)) {
+			if (p_gcd.is_one()) {
 				// a(x) = p(x)^n, gcd(p, b) = 1 ==> gcd(a, b) = 1
 				if (ca != nullptr)
 					*ca = a;
 				if (cb != nullptr)
 					*cb = b;
 				return _ex1;
-			} else {
-				// a(x) = g(x)^n A(x)^n, b(x) = g(x) B(x) ==> gcd(a, b) = g(x) gcd(g(x)^(n-1) A(x)^n, B(x))
-				return p_gcd*gcdpoly(power(p_gcd, exp_a-1)*power(p_co, exp_a), bpart_co, ca, cb, false);
 			}
+                        // a(x) = g(x)^n A(x)^n, b(x) = g(x) B(x) ==> gcd(a, b) = g(x) gcd(g(x)^(n-1) A(x)^n, B(x))
+                        return p_gcd * gcdpoly(power(p_gcd, exp_a-1)*power(p_co, exp_a),
+                                        bpart_co, ca, cb, false);
+			
 		} // is_exactly_a<power>(b)
 
 	} else if (is_exactly_a<power>(b)) {
@@ -506,31 +505,33 @@ factored_b:
 		ex p_co, apart_co;
 		const ex& exp_b(b.op(1));
 		ex p_gcd = gcdpoly(a, p, &apart_co, &p_co, false);
-		if (p_gcd.is_equal(_ex1)) {
+		if (p_gcd.is_one()) {
 			// b=p(x)^n, gcd(a, p) = 1 ==> gcd(a, b) == 1
 			if (ca != nullptr)
 				*ca = a;
 			if (cb != nullptr)
 				*cb = b;
 			return _ex1;
-		} else {
-			// there are common factors:
-			// a(x) = g(x) A(x), b(x) = g(x)^n B(x)^n ==> gcd = g(x) gcd(g(x)^(n-1) A(x)^n, B(x))
+		} 
+                // there are common factors:
+                // a(x) = g(x) A(x), b(x) = g(x)^n B(x)^n ==> gcd = g(x) gcd(g(x)^(n-1) A(x)^n, B(x))
 
-			return p_gcd*gcdpoly(apart_co, power(p_gcd, exp_b-1)*power(p_co, exp_b), ca, cb, false);
-		} // p_gcd.is_equal(_ex1)
+                return p_gcd * gcdpoly(apart_co,
+                                power(p_gcd, exp_b-1)*power(p_co, exp_b),
+                                ca, cb, false);
+		// p_gcd.is_equal(_ex1)
 	}
 
 
         ex_int_map map;
         exvector revmap;
         map.insert(std::make_pair(symbol_E, 1));
-        revmap.push_back(exp(1));
+        revmap.emplace_back(exp(1));
         On(SW_RATIONAL);
         setCharacteristic(0);
         power_ocvector_map pomap;
-        ex aa = a.subs(exp(wild()) == pow(symbol_E, wild()));
-        ex bb = b.subs(exp(wild()) == pow(symbol_E, wild()));
+        ex aa = a.subs(exp(wild()) == pow(symbol_E, wild())).expand();
+        ex bb = b.subs(exp(wild()) == pow(symbol_E, wild())).expand();
         aa.collect_powers(pomap);
         bb.collect_powers(pomap);
 //        Log(pomap,"pomap");
@@ -603,7 +604,7 @@ bool factorpoly(const ex& the_ex, ex& res_prod)
         ex_int_map map;
         exvector revmap;
         map.insert(std::make_pair(symbol_E, 1));
-        revmap.push_back(exp(1));
+        revmap.emplace_back(exp(1));
 
         On(SW_RATIONAL);
         power_ocvector_map pomap;
@@ -656,7 +657,7 @@ ex resultantpoly(const ex & ee1, const ex & ee2, const ex & s)
         ex_int_map map;
         exvector revmap;
         map.insert(std::make_pair(symbol_E, 1));
-        revmap.push_back(exp(1));
+        revmap.emplace_back(exp(1));
         On(SW_RATIONAL);
         setCharacteristic(0);
         power_ocvector_map pomap;
