@@ -53,7 +53,7 @@ static void check_poly_ccoeff_zero(const flint_series_t& fp)
 {
         if (fp.offset > 0)
                 return;
-        else if (fp.offset < 0)
+        if (fp.offset < 0)
                 throw flint_error();
         fmpq_t c;
         fmpq_init(c);
@@ -269,7 +269,7 @@ static bool rational_ex_f;
 // The helper uses recurrence to check that all numerics are from QQ,
 // that there is not more than one symbol, no constants, and all
 // function serial numbers are in the funcmap keys.
-static bool unhandled_elements_in(ex the_ex, const symbol& symb)
+static bool unhandled_elements_in(const ex& the_ex, const symbol& symb)
 {
         if (is_exactly_a<constant>(the_ex))
                 return true;
@@ -282,7 +282,7 @@ static bool unhandled_elements_in(ex the_ex, const symbol& symb)
         }
         if (is_exactly_a<function>(the_ex)) {
                 rational_ex_f = false;
-                function f = ex_to<function>(the_ex);
+                const function& f = ex_to<function>(the_ex);
                 if (funcmap().find(f.get_serial()) == funcmap().end())
                         return true;
                 for (unsigned int i=0; i<f.nops(); i++)
@@ -291,7 +291,7 @@ static bool unhandled_elements_in(ex the_ex, const symbol& symb)
                 return false;
         }
         if (is_exactly_a<power>(the_ex)) {
-                power pow = ex_to<power>(the_ex);
+                const power& pow = ex_to<power>(the_ex);
                 if (not is_exactly_a<numeric>(pow.op(1)))
                         rational_ex_f = false;
                 return (unhandled_elements_in(pow.op(0), symb)
@@ -310,7 +310,7 @@ static bool unhandled_elements_in(ex the_ex, const symbol& symb)
         return true;
 }
 
-bool useries_can_handle(ex the_ex, const symbol& s)
+bool useries_can_handle(const ex& the_ex, const symbol& s)
 {
         rational_ex_f = true;
         bool ok = (not unhandled_elements_in(the_ex, s));
@@ -340,7 +340,7 @@ class ldegree_error : public std::runtime_error {
 // series computation the precision may have to be increased. This
 // is the case if we encounter an add in the treewalk. If not we
 // can exactly determine the low degree.
-static int low_series_degree(ex the_ex) {
+static int low_series_degree(const ex& the_ex) {
         static std::unordered_set<unsigned int> funcset {{
                 sin_SERIAL::serial,
                 tan_SERIAL::serial,
@@ -358,7 +358,7 @@ static int low_series_degree(ex the_ex) {
         if (is_exactly_a<symbol>(the_ex))
                 return 1;
         if (is_exactly_a<function>(the_ex)) {
-                function f = ex_to<function>(the_ex);
+                const function& f = ex_to<function>(the_ex);
                 unsigned int ser = f.get_serial();
                 if (ser == log_SERIAL::serial)
                         return 1;
@@ -372,10 +372,10 @@ static int low_series_degree(ex the_ex) {
                 return low_series_degree(f.op(0));
         }
         if (is_exactly_a<power>(the_ex)) {
-                power pow = ex_to<power>(the_ex);
+                const power& pow = ex_to<power>(the_ex);
                 ex expo = pow.op(1);
                 if (is_exactly_a<numeric>(expo)) {
-                        numeric n = ex_to<numeric>(expo);
+                        const numeric& n = ex_to<numeric>(expo);
                         if (n.is_integer())
                                 return (low_series_degree(pow.op(0))
                                       * n.to_int());
@@ -396,7 +396,7 @@ static int low_series_degree(ex the_ex) {
         return 0;
 }
 
-ex useries(ex the_ex, const symbol& x, int order, unsigned options)
+ex useries(const ex& the_ex, const symbol& x, int order, unsigned options)
 {
         if (order <= 0)
                 // send residues to the old code
@@ -412,7 +412,7 @@ ex useries(ex the_ex, const symbol& x, int order, unsigned options)
 
         epvector epv;
         if (ldeg >= order) {
-                epv.push_back(expair(Order(_ex1), order));
+                epv.emplace_back(Order(_ex1), order);
                 return pseries(relational(x,_ex0), epv);
         }
 
@@ -446,11 +446,11 @@ ex useries(ex the_ex, const symbol& x, int order, unsigned options)
                         mpq_init(gc);
                         fmpq_get_mpq(gc, c);
                         numeric nc(gc); // numeric clears gc
-                        epv.push_back(expair(nc, numeric(n + fp.offset)));
+                        epv.emplace_back(nc, numeric(n + fp.offset));
                 }
                 fmpq_clear(c);
         }
-        epv.push_back(expair(Order(_ex1), order));
+        epv.emplace_back(Order(_ex1), order);
         return pseries(relational(x,_ex0), epv);
 }
 
@@ -527,7 +527,7 @@ void power::useries(flint_series_t& fp, int order) const
                 fmpq_poly_exp_series(fp.ft, fp.ft, order);
                 return;
         }
-        numeric nexp = ex_to<numeric>(exponent);
+        const numeric& nexp = ex_to<numeric>(exponent);
         if (nexp.is_mpq()) {
                 int num = nexp.numer().to_int();
                 int den = nexp.denom().to_int();
@@ -578,7 +578,7 @@ void power::useries(flint_series_t& fp, int order) const
                 fmpq_poly_truncate(fp.ft, fp.offset + order + 2);
                 return;
         }
-        else if (expint < 0) {
+        if (expint < 0) {
                 if (fmpq_poly_is_zero(fp1.ft))
                         throw flint_error();
                 if (ldeg) {
